/* CAUnreal3.2 Module */
/* (C) Tom McIntyre, Dec 2001 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef _WIN32
#include <io.h>
#endif
#include <fcntl.h>
#include "h.h"
#ifdef STRIPBADWORDS
#include "badwords.h"
#endif
#ifdef _WIN32
#include "version.h"
#endif

DLLFUNC int m_relay(aClient *cptr, aClient *sptr, int parc, char *parv[]);
DLLFUNC int m_grelay(aClient *cptr, aClient *sptr, int parc, char *parv[]);
DLLFUNC int m_glag(aClient *cptr, aClient *sptr, int parc, char *parv[]);

#define MSG_RELAY		"RELAY"
#define TOK_RELAY		"RL"
#define MSG_GRELAY		"GRELAY"
#define TOK_GRELAY		"GR"
#define MSG_GLAG		"GLAG"
#define TOK_GLAG		"GL"

#ifndef DYNAMIC_LINKING
ModuleHeader m_relay_Header
#else
#define m_relay_Header Mod_Header
ModuleHeader Mod_Header
#endif
  = {
	"Relay",						/* Name of module */
	"m_relay.c 1.02 2002/08/02 Chawmp",		     /* Version */
	"For transmitting command strings across the network",  /* Short description of module */
	"3.2-b5",
	NULL
    };

/* This is called on module init, before Server Ready */
#ifdef DYNAMIC_LINKING
DLLFUNC int	Mod_Init(int module_load)
#else
int    m_relay_Init(int module_load)
#endif
{
	add_Command(MSG_RELAY, TOK_RELAY, m_relay, MAXPARA);
	add_Command(MSG_GRELAY, TOK_GRELAY, m_grelay, MAXPARA);
	add_Command(MSG_GLAG, TOK_GLAG, m_glag, MAXPARA);
	return MOD_SUCCESS;
}

/* Is first run when server is 100% ready */
#ifdef DYNAMIC_LINKING
DLLFUNC int	Mod_Load(int module_load)
#else
int    m_relay_Load(int module_load)
#endif
{
	return MOD_SUCCESS;
}

/* Called when module is unloaded */
#ifdef DYNAMIC_LINKING
DLLFUNC int	Mod_Unload(int module_unload)
#else
int m_relay_Unload(int module_unload)
#endif
{
	if (del_Command(MSG_RELAY, TOK_RELAY, m_relay) < 0)
	{
		sendto_realops("Failed to delete command RELAY when unloading %s",
		m_relay_Header.name);
	}
	if (del_Command(MSG_GRELAY, TOK_GRELAY, m_grelay) < 0)
	{
		sendto_realops("Failed to delete command GRELAY when unloading %s",
		m_relay_Header.name);
	}
	if (del_Command(MSG_GLAG, TOK_GLAG, m_glag) < 0)
	{
		sendto_realops("Failed to delete command GLAG when unloading %s",
		m_relay_Header.name);
	}
	return MOD_SUCCESS;
}

/*
** m_relay
**      parv[0] = sender
**      parv[1] = server pattern to relay to
**      parv[2] = string to relay
*/
DLLFUNC int m_relay(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	Link *lp;

	if (!(IsNetAdmin(sptr) || IsServer(cptr) || IsULine(sptr))) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return -1;
	}

	if (parc < 3 || *parv[1] == '\0')
	{
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "RELAY");
		return -1;
	}

	if (!IsServer(cptr)) {
		sendto_serv_butone(&me,
		    ":%s GLOBOPS :%s is RELAYing [%s] to %s",
		    me.name, sptr->name, parv[2], parv[1]);
		sendto_realops
		    ("%s is RELAYing [%s] to %s",
		    parv[0], parv[2], parv[1]);
	}

	if (hunt_server(cptr, sptr, ":%s RELAY %s :%s", 1, parc, parv) == HUNTED_ISME) {
		parse(sptr, parv[2], (parv[2] + strlen(parv[2])));
	}

	return 0;
}

/*
** m_grelay
**      parv[0] = sender
**      parv[1] = string to relay
*/
DLLFUNC int m_grelay(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	Link *lp;

	if (!(IsNetAdmin(sptr) || IsULine(sptr))) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return -1;
	}

	if (parc < 2 || *parv[1] == '\0')
	{
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "GRELAY");
		return -1;
	}

	sendto_serv_butone(&me,
	    ":%s GLOBOPS :%s is GRELAYing [%s]",
	    me.name, sptr->name, parv[1]);
	sendto_realops
	    ("%s is GRELAYing [%s]",
	    parv[0], parv[1]);


	parv[2] = parv[1];
	parc++;

	for (lp = Servers; lp; lp = lp->next) {
		parv[1] = lp->value.cptr->name;
		if (hunt_server(cptr, sptr, ":%s RELAY %s :%s", 1, parc, parv) == HUNTED_ISME) {
			parse(sptr, parv[2], (parv[2] + strlen(parv[2])));
			break;
		}
	}

	return 0;
}

/*
** m_glag
**      parv[0] = sender
*/
DLLFUNC int m_glag(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	Link *lp;

	if (!IsOper(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return -1;
	}

	for (lp = Servers; lp; lp = lp->next) {
		parv[1] = lp->value.cptr->name;
		if (hunt_server(cptr, sptr, ":%s RELAY %s :LAG", 1, 2, parv) == HUNTED_ISME) {
			parv[2] = strdup("LAG");
			parse(sptr, parv[2], (parv[2] + 3));
			free(parv[2]);
			break;
		}
	}

	return 0;
}
